@@ -3,16 +3,20 @@ import ast
 import sys
 from functools import singledispatch
 import inspect
-from typing import Callable
+from typing import Callable, NewType
 
 from .symbol import Symbol
 from .expression import Expr, Head, symbol
 
+Operator = NewType("Operator", type)
+NoneType = type(None)
+
 AST_BINOP_MAP = {
     ast.Add: Symbol("+"),
     ast.Sub: Symbol("-"),
     ast.Mult: Symbol("*"),
     ast.Div: Symbol("/"),
+    ast.Mod: Symbol("%"),
     ast.Eq: Symbol("=="),
     ast.NotEq: Symbol("!="),
     ast.Gt: Symbol(">"),
@@ -33,8 +37,6 @@ AST_BINOP_MAP = {
     ast.Or: Symbol("or"),
 }
 
-NoneType = type(None)
-
 def parse(source: str | Callable) -> Expr | Symbol:
     """
     Convert Python code string into Expr/Symbol objects.
@@ -65,7 +67,7 @@ def _(ast_object: ast.Expr):
     return from_ast(ast_object.value)
 
 
-if sys.version_info[:2] <= (3, 7):
+if sys.version_info <= (3, 7):
     @from_ast.register
     def _(ast_object: ast.Num):
         return symbol(ast_object.n)
@@ -181,7 +183,7 @@ def _(ast_object: ast.If):
 @from_ast.register
 def _(ast_object: ast.For):
     head = Head.for_
-    top = Expr(Head.binop, [Symbol("in"), from_ast(ast_object.target), from_ast(ast_object.iter)])
+    top = Expr(Head.binop, [Symbol.var("in", Operator), from_ast(ast_object.target), from_ast(ast_object.iter)])
     block = from_ast(ast_object.body)
     return Expr(head, [top, block])
 
@@ -221,7 +223,7 @@ def _(ast_object: ast.AnnAssign):
 
 @from_ast.register
 def _(ast_object: ast.Pass):
-    return Symbol("pass") # Should it be Expr(Head.pass_, [Symbol("pass")]) ??
+    return Symbol.var("pass") # Should it be Expr(Head.pass_, [Symbol("pass")]) ??
     
 @from_ast.register
 def _(ast_object: ast.Return):
