@@ -8,7 +8,7 @@ import inspect
 from typing import Callable, Iterable, Iterator, Any, TypedDict, Union, overload, TypeVar, NamedTuple
 from types import ModuleType
 
-from .ast import parse
+from .ast import parse, Operator
 from .expression import Head, Expr, symbol, EXEC
 from .symbol import Symbol
 
@@ -17,11 +17,13 @@ _NON_RECORDABLE = ("__new__", "__class__", "__repr__", "__getattribute__", "__di
 
 _INHERITABLE = ("__module__", "__name__", "__qualname__", "__doc__", "__annotations__")
 
+
 BINOP_MAP = {
     "__add__": Symbol("+"),
     "__sub__": Symbol("-"),
     "__mul__": Symbol("*"),
     "__div__": Symbol("/"),
+    "__mod__": Symbol("%"),
     "__eq__": Symbol("=="),
     "__neq__": Symbol("!="),
     "__gt__": Symbol(">"),
@@ -36,6 +38,17 @@ BINOP_MAP = {
     "__xor__": Symbol("^")
 }
 
+BUILTIN_MAP = {
+    "__hash__": Symbol("hash"),
+    "__len__": Symbol("len"),
+    "__str__": Symbol("str"),
+    "__repr__": Symbol("repr"),
+    "__bool__": Symbol("bool"),
+    "__float__": Symbol("float"),
+    "__int__": Symbol("int"),
+    "__format__": Symbol("format"),
+}
+
 class MacroFlags(NamedTuple):
     """
     This immutable struct gives the infomation of what kind of expressions will be
@@ -80,6 +93,7 @@ class Macro(Expr, MutableSequence[Expr]):
         self.active = True
         self._callbacks = []
         self._flags = MacroFlags(**flags)
+        self._last_setval: Expr = None
     
     @property
     def callbacks(self):
@@ -175,6 +189,7 @@ class Macro(Expr, MutableSequence[Expr]):
             A function that will called after new expression is appended. Must take an expression
             or an expression with the last returned value as inputs.
         """        
+        # TODO: record called twice
         kwargs = dict(macro=self, 
                       returned_callback=returned_callback, 
                       record_returned=self._flags.Return
@@ -340,13 +355,13 @@ def update_namespace(obj: MacroMixin, namespace: Symbol | Expr) -> None:
             attr.namespace = namespace
         elif isinstance(attr, MacroMixin):
             update_namespace(attr, new)
-    
+
+@Symbol.register_type(lambda o: symbol(o.obj))
 class mObject:
     """
     Abstract class for macro recorder equipped objects.
     """    
-    obj: Any
-    def __init__(self, obj: Any, macro: Macro, returned_callback: MetaCallable = None, 
+    def __init__(self, obj, macro: Macro, returned_callback: MetaCallable = None, 
                  namespace: Symbol|Expr = None, record_returned: bool = True) -> None:
         self.obj = obj
         
@@ -372,13 +387,11 @@ class mObject:
             
         self.namespace = namespace
         
-        self.macro = macro
+        self._macro = macro
         for name in _INHERITABLE:
             if hasattr(self.obj, name):
                 setattr(self, name, getattr(self.obj, name))
-        
-        self._last_setval: Expr = None
-    
+            
     def to_namespace(self, obj: Any) -> Symbol | Expr:
         """
         Return the expression of ``obj`` in the correct name space.
@@ -388,14 +401,33 @@ class mObject:
             return sym
         else:
             return Expr(Head.getattr, [self.namespace, sym])
+    
+    @property
+    def macro(self) -> Macro:
+        return self._macro
 
-Symbol.register_type(mObject, lambda o: symbol(o.obj))
+_GET = Symbol.var("__get__", Callable)
+_SET = Symbol.var("__set__", Callable)
+_DELETE = Symbol.var("__delete__", Callable)
 
 class mCallable(mObject):
     obj: Callable
     def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
                  namespace: Symbol|Expr = None, record_returned: bool = True):
         super().__init__(function, macro, returned_callback, namespace, record_returned)
+        if self.__name__ == "__get__":
+            def make_expr(*args, **kwargs):
+                return Expr.parse_method(args[0], _GET, (args[1], args[2]), {})
+        elif self.__name__ == "__set__":
+            def make_expr(*args, **kwargs):
+                return Expr.parse_method(args[0], _SET, (args[1], args[2]), {})
+        elif self.__name__ == "__delete__":
+            def make_expr(*args, **kwargs):
+                return Expr.parse_method(args[0], _DELETE, (args[1]), {})
+        else:
+            def make_expr(*args, **kwargs):
+                return Expr.parse_call(self.to_namespace(self.obj), args, kwargs)
+        self._make_expr = make_expr
     
     @property
     def __signature__(self):
@@ -405,13 +437,14 @@ class mCallable(mObject):
             return inspect.signature(self.obj)
     
     def __call__(self, *args, **kwargs):
-        with self.macro.blocked():
+        # TODO: This function will be called if __get__ method is the origin, like A.__get__(...).
+        with self._macro.blocked():
             out = self.obj(*args, **kwargs)
-        if self.macro.active:
-            expr = Expr.parse_call(self.to_namespace(self.obj), args, kwargs)
+        if self._macro.active:
+            expr = self._make_expr(*args, **kwargs)
             line = self.returned_callback(expr, out)
-            self.macro.append(line)
-            self._last_setval = None
+            self._macro.append(line)
+            self._macro._last_setval = None
         return out
     
 class mFunction(mCallable):
@@ -421,82 +454,87 @@ class mFunction(mCallable):
     def __init__(self, function: Callable, macro: Macro, returned_callback: MetaCallable = None,
                  namespace: Symbol|Expr = None, record_returned: bool = True):
         super().__init__(function, macro, returned_callback, namespace, record_returned)
-        self._method_type = self._make_method_type()
+        self._method_type = None
             
     def _make_method_type(self):
-        # TODO: __get__ etc. is not recorded correctly
         fname = getattr(self.obj, "__name__", None)
         if fname == "__init__":
             def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_init(self.to_namespace(obj), obj.__class__, args, kwargs)
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname == "__call__":
             def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_call(self.to_namespace(obj), args, kwargs)
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname == "__getitem__":
             def make_expr(obj: _O, *args):
                 expr = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname == "__getattr__":
             def make_expr(obj: _O, *args):
                 expr = Expr(Head.getattr, [self.to_namespace(obj), Symbol(args[0])])
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname == "__setitem__":
             def make_expr(obj: _O, *args):
                 target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.assign, [target, args[1]])
-                if self._last_setval == target:
-                    self.macro.pop(-1)
+                if self._macro._last_setval == target:
+                    self._macro.pop(-1)
                 else:
-                    self._last_setval = target
+                    self._macro._last_setval = target
                 return expr
         elif fname == "__setattr__":
             def make_expr(obj: _O, *args):
                 target = Expr(Head.getattr, [self.to_namespace(obj), Symbol(args[0])])
                 expr = Expr(Head.assign, [target, args[1]])
-                if self._last_setval == target:
-                    self.macro.pop(-1)
+                if self._macro._last_setval == target:
+                    self._macro.pop(-1)
                 else:
-                    self._last_setval = target
+                    self._macro._last_setval = target
                 return expr
         elif fname == "__delitem__":
             def make_expr(obj: _O, *args):
                 target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.del_, [target])
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname == "__delattr__":
             def make_expr(obj: _O, *args):
                 target = Expr(Head.getattr, [self.to_namespace(obj), args[0]])
                 expr = Expr(Head.del_, [target])
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         elif fname in BINOP_MAP.keys():
             op = BINOP_MAP[fname]
             def make_expr(obj: _O, *args):
                 expr = Expr(Head.binop, [op, self.to_namespace(obj), args[0]])
-                self._last_setval = None
+                self._macro._last_setval = None
+                return expr
+        elif fname in BUILTIN_MAP.keys():
+            f = BUILTIN_MAP[fname]
+            def make_expr(obj: _O, *args):
+                expr = Expr(Head.call, [f, self.to_namespace(obj)] + list(args))
+                self._macro._last_setval = None
                 return expr
         else:
             def make_expr(obj: _O, *args, **kwargs):
                 expr = Expr.parse_method(self.to_namespace(obj), self.obj, args, kwargs)
-                self._last_setval = None
+                self._macro._last_setval = None
                 return expr
         
         @wraps(self.obj)
         def method(obj: _O, *args, **kwargs):
-            with self.macro.blocked():
+            with self._macro.blocked():
                 out = self.obj(obj, *args, **kwargs)
-            if self.macro.active:
+            if self._macro.active:
                 expr = make_expr(obj, *args, **kwargs)
                 if expr is not None:
                     line = self.returned_callback(expr, out)
-                self.macro.append(line)
+                self._macro.append(line)
             return out
 
         return method
@@ -509,6 +547,8 @@ class mFunction(mCallable):
         if obj is None:
             return self.obj
         else:
+            if self._method_type is None:
+                self._method_type = self._make_method_type()
             return mMethod(self._method_type, obj)
 
 class mMethod(partial, mCallable):
@@ -565,13 +605,13 @@ class mProperty(mObject, property):
         key = Symbol(fget.__name__)
         @wraps(fget)
         def getter(obj):
-            with self.macro.blocked():
+            with self._macro.blocked():
                 out = fget(obj)
-            if self.macro.active:
+            if self._macro.active:
                 expr = Expr(Head.getattr, [self.to_namespace(obj), key])
                 expr = self.returned_callback(expr, out)
-                self.macro.append(expr)
-                self._last_setval = None
+                self._macro.append(expr)
+                self._macro._last_setval = None
             return out
         return self.obj.getter(getter)
         
@@ -581,18 +621,18 @@ class mProperty(mObject, property):
         key = Symbol(fset.__name__)
         @wraps(fset)
         def setter(obj, value):
-            with self.macro.blocked():
+            with self._macro.blocked():
                 out = fset(obj, value)
-            if self.macro.active:
+            if self._macro.active:
                 target = Expr(Head.getattr, [self.to_namespace(obj), key])
                 expr = Expr(Head.assign, [target, value])
-                if self._last_setval == target:
-                    self.macro.pop(-1)
+                if self._macro._last_setval == target:
+                    self._macro.pop(-1)
                 else:
-                    self._last_setval = target
+                    self._macro._last_setval = target
                 
                 line = self.returned_callback(expr, out)
-                self.macro.append(line)
+                self._macro.append(line)
                     
             return out
         
@@ -604,14 +644,14 @@ class mProperty(mObject, property):
         key = Symbol(fdel.__name__)
         @wraps(fdel)
         def deleter(obj):
-            with self.macro.blocked():
+            with self._macro.blocked():
                 out = fdel(obj)
-            if self.macro.active:
+            if self._macro.active:
                 target = Expr(Head.getattr, [self.to_namespace(obj), key])
                 expr = Expr(Head.del_, [target])
                 line = self.returned_callback(expr, out)
-                self.macro.append(line)
-                self._last_setval = None
+                self._macro.append(line)
+                self._macro._last_setval = None
             return out
         
         return self.obj.deleter(deleter)
@@ -645,23 +685,23 @@ class mModule(mObject):
                 raise ValueError(f"No function or submodule named '{key}'.")
         
         if _type == "module":
-            mmod = mModule(attr, self.macro, self.returned_callback, self.to_namespace(self.obj))
+            mmod = mModule(attr, self._macro, self.returned_callback, self.to_namespace(self.obj))
             setattr(self, key, mmod)
             return mmod
             
         @wraps(attr)
         def mfunc(*args, **kwargs):
-            with self.macro.blocked():
+            with self._macro.blocked():
                 out = attr(*args, **kwargs)
-            if self.macro.active:
+            if self._macro.active:
                 if isinstance(attr, type):
                     cls = Expr(Head.getattr, [self.to_namespace(self.obj), Symbol(out.__class__.__name__)])
                     expr = Expr.parse_init(out, cls, args, kwargs)    
                 else:
                     expr = Expr.parse_method(self.to_namespace(self.obj), attr, args, kwargs)
                 expr = self.returned_callback(expr, out)
-                self.macro.append(expr)
-                self._last_setval = None
+                self._macro.append(expr)
+                self._macro._last_setval = None
             return out
         return mfunc
     
