@@ -88,12 +88,13 @@ def test_class():
     
     a = A(4)
     assert a.value_str == "4"
+    a.value_str = 4 # This line should not be recorded.
     a.value_str = 5
     assert a.getval() == 5
     A.set_class_var(10)
     assert a["key"] == "key"
+    a["a"] = False # This line should not be recorded.
     a["a"] = True
-    hash(a)
     
     macro_str = str(macro.format([(a, Symbol("a"))]))
     assert macro_str == \
@@ -103,11 +104,10 @@ def test_class():
         "a.getval()\n" \
         "A.set_class_var(10)\n"\
         "a['key']\n" \
-        "a['a'] = True\n" \
-        "a.__hash__()"
+        "a['a'] = True"
 
 def test_register_type():
-    from macrokit import register_type
+    from macrokit import register_type, symbol
     import numpy as np
     
     macro = Macro()
@@ -120,4 +120,78 @@ def test_register_type():
     
     out = double(np.arange(3))
     macro_str = str(macro.format([(out, Symbol("out"))]))
-    assert macro_str == "out = double([0, 1, 2])"
\ No newline at end of file
+    assert macro_str == "out = double([0, 1, 2])"
+    
+    @register_type(np.ndarray)
+    def _(arr: np.ndarray):
+        return f"array{arr.shape}"
+    assert str(symbol(np.zeros((4,6)))) == "array(4, 6)"
+    
+    @register_type(lambda e: e.name)
+    class T:
+        def __init__(self):
+            self.name = "t"
+    assert str(symbol(T())) == "t"
+
+def test_symbol_var():
+    sym_x = Symbol("x")
+    sym_y = Symbol("y")
+    var_y0 = Symbol.var("y")
+    var_y1 = Symbol.var("y")
+    var_y2 = Symbol.var("y")
+    var_y3 = Symbol.var("y")
+    
+    assert sym_x == sym_x
+    assert sym_x != sym_y
+    assert sym_y != var_y0
+    assert var_y0 == var_y1 and var_y1 == var_y2 and var_y2 == var_y3
+
+code1 = """
+a = np.arange(12)
+for i in a:
+    print(a)
+    if i % 3 == 0:
+        print("3n")
+"""
+
+code2 = """
+def g(a: int = 4):
+    return a - 1, a + 1
+"""
+
+    
+def test_parsing():
+    from macrokit import parse
+    parse(code1)
+    parse(code2)
+
+def test_special_methods():
+    macro = Macro(flags={"Return": False})
+    @macro.record
+    class A:
+        def __len__(self):
+            return 0
+        def __bool__(self):
+            return True
+        def __int__(self):
+            return 0
+        def __float__(self):
+            return 0.0
+        def __str__(self):
+            return ""
+    
+    a = A()
+    len(a)
+    bool(a)
+    int(a)
+    float(a)
+    str(a)
+    
+    macro_str = str(macro.format([(a, Symbol("a"))]))
+    assert macro_str == \
+        "a = A()\n" \
+        "len(a)\n" \
+        "bool(a)\n" \
+        "int(a)\n" \
+        "float(a)\n" \
+        "str(a)"
\ No newline at end of file
